% Code to compute the Deterministic and Risky Steady State of the model
%(c) Bigio, Nuno & Passadore model
% This version codes Galo's solution using standard finite difference methods.
% The Code Solves for the Risky Steady State with default
% the code computes the risky-ss, simple steady state or the HRSS
% the HRSS always requires the pre-computation of the RSS as an initial
% guess

%% Structure of Results
% endogenous objects: 
%   - prices: psi
%   - values: v
%   - debt profile: f
%   - issuances: iota
%   - consumption: c
%   - discount: r
% for multiple shocks:
%   - structures always have N_rxN_y (rates always first in indexing)
% -----------------------------------
% results: structure that saves the following objects
% -----------------------------------
% any object with _n suffices: 
%       - length is thus N_t+1 in all cases
%       - initial point is ss, rss, or hrss
%       - all additional values are transition points
%       
% -------------------------------------
% steady:
% ---------------------------------------
% steady state objects only
% 
% ---------------------------------------
% rss: strcuture (rss) objects 
% ---------------------------------------
%       - strictly rss objects (no path)
%       - can be steady state or not, depending on whether shock is
%       considered.
%       - if you run hrss, then will be (only) the guess we used 
%
% ---------------------------------------
% hrss: hrss objects
% ---------------------------------------
%       - strictly hrss objects (no path)
%       - not computed unless told so 
% ---------------------------------------
% hr: hr objects
% ---------------------------------------
%       - store hr loop objects
%       - when we loop over prices in the hr-ss, we have to store objects
%       to pass them on.
% paths: stores path objects, only to be used internally...

% plot conditions
plotit=0;

%% Pre-Allocation
time_preallocate;

%% Paths
% Path for the Short Rate and output
exo_path_construct;

% Autarky Values
autarkyV_construct;
parameters.V_a = V_a(:);
%parameters.V_a = kron(ones(N_r,1),V_a)     ;

%% Save Exogenous Paths
paths.r_bar_n = r_bar_n;
paths.y_n     = y_n    ;

%% Step 1: Bond Prices: Steady State and RSS computation
% Compute price path: RSS 
psi_ss         = solve_HJB_ss(r_bar_ss,parameters); % compute steady-state prices
steady.psi_ss  = psi_ss                           ; % append steady state price

% Compute price paths for rss
compute_psi_rss_paths                       ; % compute RSS price path, store in objects
paths.psi_n    = psi_n                      ; % save path

%% Step 2: Value functions in Steady State 
steady=solve_steady(psi_ss,parameters)            ; % compute steady-state objects

%% Step 2: Risky-Steady State 
path_out             = solve_rss_path(rss,steady,paths,parameters); % [compute entire transition, given steady-state objects and rss.price]
rss.psi_rss          = path_out.psi_rss  ;
rss.a                = path_out.a_rss    ;
rss.Phi              = path_out.Phi      ;
rss.psi_J_rss        = path_out.psi_J_rss;

aux=[]; ii=0;
for rr=1:N_r
    for yy=1:N_y
        ii=ii+1;
        aux=[aux path_out.v_n(:,1,ii)]; 
    end 
end

path_out.rss.J_v_rss = aux./(path_out.rss.v_rss*ones(1,N_r*N_y));
results.psi_rss      = path_out.psi_rss;
results.yield_rss    = parameters.delta-log(results.psi_rss)./(tau');
if strcmp(parameters.mode,'rss')||strcmp(parameters.mode,'0');
    results.f_n     = path_out.f_n          ;
    results.f_rss   = path_out.f_out(:,1)   ;
    results.iota_n  = path_out.iota_n       ;
    results.iota_rss= path_out.iota_out(:,1);
    results.v_n     = path_out.v_n          ; 
    results.v_rss   = path_out.v_out(:,1)   ; 
    results.r_n     = path_out.r_n          ; 
    results.c_rss   = path_out.c_out(:,1)   ; 
    results.c_n     = path_out.c_n          ;
end

%% Step 2: Homethetic Risk path [SB: change path structure.]
% saving objects as initial guesses.

% if we need a guess for the price system, update here

% computing paths...

if strcmp(parameters.mode,'hr_trans');    
    % Construct and move to new function...
    %hr.psi_J_rss 
    Phi=rss.Phi; % Intial guess, no default
    
    %   compute_psi_hrss_paths;
    hr=compute_psi_hrss_paths(rss,Phi,paths,parameters);
    hr.v_hrss       = path_out.rss.v_rss;
    hr.r_hrss       = path_out.rss.r_rss;
    hr.iota_hrss    = path_out.rss.iota_rss;
    hr.c_hrss       = path_out.rss.c_rss;
    hr.f_hrss       = path_out.rss.f_rss;
    path_out.psi_n  = hr.psi_hr_n;
    path_out.rss.f_rss = f_init; % Substitute initial condtion...
    
    % compute equilibrium objects...
    path_out_hr        = solve_hr_path(hr,path_out,parameters); %     

    % not sure we are using these dudes....
    path_out_hr.psi_n=hr.psi_hr_n;
    path_out_hr.y_n     = y_n    ;

    %% Step 3: Pack Results
    % results.yield_rss = parameters.delta-log(results.psi_rss)./(tau');
    results.psi_hrss    = path_out_hr.psi_hrss     ;
    results.f_n         = path_out_hr.f_n          ;
    results.f_hrss      = path_out_hr.f_out(:,1)   ;
    results.iota_n      = path_out_hr.iota_n       ;
    results.iota_hrss   = path_out_hr.iota_out(:,1);
    results.v_n         = path_out_hr.v_n          ; 
    results.v_hrss      = path_out_hr.v_out(:,1)   ; 
    results.r_n         = path_out_hr.r_n          ; 
    results.c_hrss      = path_out_hr.c_out(:,1)   ; 
    results.c_n         = path_out_hr.c_n          ;
    results.c_out       = path_out_hr.c_out        ;
 clear paths;    
end


if strcmp(parameters.mode,'hr');    
    % Construct and move to new function...
    %hr.psi_J_rss 
     Phi=rss.Phi; % Intial guess, no default
    
    %   compute_psi_hrss_paths;
    hr=compute_psi_hrss_paths(rss,Phi,paths,parameters);
    hr.v_hrss       = path_out.rss.v_rss;
    hr.r_hrss       = path_out.rss.r_rss;
    hr.iota_hrss    = path_out.rss.iota_rss;
    hr.c_hrss       = path_out.rss.c_rss;
    hr.f_hrss       = path_out.rss.f_rss;
    path_out.psi_n  = hr.psi_hr_n;

    % compute equilibrium objects...
    path_out_hr        = solve_hr_path(hr,path_out,parameters); %     

    % not sure we are using these dudes....
    path_out_hr.psi_n=hr.psi_hr_n;
    path_out_hr.y_n     = y_n    ;

    %% Step 3: Pack Results
    % results.yield_rss = parameters.delta-log(results.psi_rss)./(tau');
    results.psi_hrss    = path_out_hr.psi_hrss     ;
    results.f_n         = path_out_hr.f_n          ;
    results.f_hrss      = path_out_hr.f_out(:,1)   ;
    results.iota_n      = path_out_hr.iota_n       ;
    results.iota_hrss   = path_out_hr.iota_out(:,1);
    results.v_n         = path_out_hr.v_n          ; 
    results.v_hrss      = path_out_hr.v_out(:,1)   ; 
    results.r_n         = path_out_hr.r_n          ; 
    results.c_hrss      = path_out_hr.c_out(:,1)   ; 
    results.c_n         = path_out_hr.c_n          ;
    results.c_out       = path_out_hr.c_out        ;
end
clear paths; 